<?php

namespace FiloBlu\Flow\Model\Channel\In;

/**
 * Class AbstractModel
 * @package FiloBlu\Flow\Model\Channel\In
 */
abstract class AbstractModel
{
    public $map;
    protected $_name;
    protected $_data;
    protected $_connector;
    protected $_parser;
    protected $_config;
    protected $_configModel;
    protected $_file;
    protected $_insertQueue = [];
    protected $_tablename;

    /**
     * @var \Monolog\Logger
     */
    protected $_logger;

    /**
     * AbstractModel constructor.
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     * @param \FiloBlu\Flow\Model\Channel\Config $channelConfig
     */
    public function __construct(
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \FiloBlu\Flow\Model\Channel\Config $channelConfig
    ) {
        $this->_logger = $loggerProvider->getLogger();
        $this->_configModel = $channelConfig;
    }

    /**
     * @param $channelname
     */
    public function init($channelname)
    {
        $this->loadConfig($channelname);
    }

    /**
     * @param $name
     * @return bool
     */
    public function loadConfig($name)
    {

        // $this->_logger->info($name);

        // $name -> DB flow_channels.name
        // unique value (string)

        $this->_configModel->load($name);

        if ($this->_configModel->initialized) {
            $this->_config = $this->_configModel->get('config');
            $this->setConnector($this->_configModel->get('connector'));
            $this->setParser($this->_configModel->get('parser'));
            return true;
        }
        return false;
    }

    /**
     * @param $data
     * @return bool
     */
    public function insertData($data)
    {
        // Genero il FLusso ITEMS
        $model = \Magento\Framework\App\ObjectManager::getInstance()->create('FiloBlu\Flow\Model' . "\\" . $this->getConfig('flow'));

        if (!$model) {
            return false;
        }

        $model->setData('meta_file', $this->getFile()->getId());

        foreach ($data as $field => $value) {
            $fieldMap = $this->map->get($field);
            if (!isset($fieldMap['skip'])) {
                $to_field = $fieldMap['to'];
                $model->setData($to_field, $value);
            }
        }

        $model->save();
    }

    /**
     * @param $key
     * @return null
     */
    public function getConfig($key)
    {
        if (isset($this->_config[$key])) {
            return $this->_config[$key];
        }
        return null;
    }

    /**
     * @return mixed
     */
    public function getFile()
    {
        return $this->_file;
    }

    /**
     * @param $file
     * @return $this
     */
    public function setFile($file)
    {
        $this->_file = $file;
        return $this;
    }

    /**
     * @return mixed
     */
    public function getName()
    {
        return $this->_name;
    }

    /**
     * @param $name
     */
    public function setName($name)
    {
        $this->_name = $name;
    }

    /**
     * @param $config
     */
    public function load($config)
    {
        $this->_data = (Array)$config;
    }

    /**
     *
     */
    public function storeData()
    {

    }

    /**
     * @return \FiloBlu\Flow\Model\Connector\AbstractConnector
     */
    public function getConnector()
    {
        return $this->_connector;
    }

    /**
     * @param $connector
     * @throws \Exception
     */
    public function setConnector($connector)
    {
        $connector_type = $connector['type'];

        $connector_class = \Magento\Framework\App\ObjectManager::getInstance()->create('FiloBlu\Flow\Model\Connector' . "\\" . ucfirst($connector_type));

        if (trim($connector_type) and ($connector_class instanceof \FiloBlu\Flow\Model\Connector\AbstractConnector)) {
            $this->_connector = $connector_class;
            $this->_connector->setConfig($connector);
        } else {
            throw new \Exception('Error in connector config');
        }
    }

    /**
     * @return mixed
     */
    public function getParser()
    {
        return $this->_parser;
    }

    /**
     * @param $parser
     * @throws \Exception
     */
    public function setParser($parser)
    {
        $parser_type = $parser['type'];

        $parser_class = \Magento\Framework\App\ObjectManager::getInstance()->create('FiloBlu\Flow\Model\Parser' . "\\" . ucfirst($parser_type));

        if (trim($parser_type) && ($parser_class instanceof \FiloBlu\Flow\Model\Parser\AbstractParser)) {
            $this->_parser = $parser_class;
            $this->_parser->setConfig($parser);
        } else {
            throw new \Exception('Error in parser config');
        }
    }

    /**
     *
     */
    public function processBeginningOfFlow()
    {

    }

    /**
     *
     */
    public function processEndOfFlow()
    {

    }

    /**
     * @return bool|mixed|void
     * @throws \Exception
     */
    public function rlist()
    {

        // $this->_logger->info(__METHOD__);

        if (!$this->_connector) {
            throw new \Exception('no connector joined to channel');
        }

        if ($this->getData('depends_folder')) {
            $depends_resources = $this->_connector->listDir($this->getData('depends_folder'));
            if (count($depends_resources) > 0) {
                return false;
            }
        }

        if ($this->getData('depends_file')) {
            $depends_resources = $this->_connector->getResources();;

            if (count($depends_resources) > 0) {
                return false;
            }
        }

        $resources = $this->_connector->getResources();

        if (!count($resources)) {
            return false;
        }

        return $resources;

    }

    /**
     * @param $key
     * @return null
     */
    public function getData($key)
    {
        if (isset($this->_data[$key])) {
            return $this->_data[$key];
        }
        return null;
    }

    /**
     * @param $file
     * @return mixed
     * @throws \Exception
     */
    public function receiveFile($file)
    {

        if (!$this->_connector) {
            throw new \Exception('no connector joined to channel');
        }

        return $this->_connector->receiveResource($file->getName());
    }

    /**
     * @param $file
     * @return bool
     * @throws \Exception
     */
    public function parse($file)
    {

        if (!$this->_connector) {
            throw new \Exception('No connector joined to channel');
        }

        if (!$this->_parser) {
            throw new \Exception('No parser joined to channel');
        }

        $this->setFile($file);
        $localdir = $this->_connector->getLocalDir();

        $this->_parser->loadMap($file->getChannel());
        $this->setMap($this->_parser->map);
        $this->_parser->setChannel($this);

        //Il Parser chiama il channel con il metodo insertData
        try {
            $this->_parser->process($localdir . DIRECTORY_SEPARATOR . $file->getName());
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }

        return true;
    }

    /**
     * @param $map
     * @return $this
     */
    public function setMap($map)
    {
        $this->map = $map;
        return $this;
    }

    /**
     *
     */
    public function processendingofFlow()
    {

    }

    /**
     * @return bool
     */
    public function commitData()
    {
        return true;
    }

}
